# Generated by Django 3.2.20 on 2023-11-22 08:24

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('moodle', '0028_auto_20231116_1826'),
        ('moodle', '0029_auto_20231106_1233'),
    ]

    operations = [
    ]
