# Generated by Django 2.2.13 on 2020-06-30 20:22

from django.db import migrations


# this field is new, there is no data in prod that we have to update
def move_data_from_marked_done_to_saved_for_later(apps, schema_editor):
   pass


def move_data_from_saved_for_later_to_marked_done(apps, schema_editor):
    pass

class Migration(migrations.Migration):

    dependencies = [
        ('enterprise', '0100_add_licensed_enterprise_course_enrollment'),
    ]

    operations = [
        migrations.RunPython(move_data_from_marked_done_to_saved_for_later, move_data_from_saved_for_later_to_marked_done)
    ]
