# Generated by Django 3.2.23 on 2023-11-30 22:39

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise', '0196_backfill_sso_marked_authorized'),
    ]

    operations = [
        migrations.AlterField(
            model_name='enterprisecustomerssoconfiguration',
            name='marked_authorized',
            field=models.BooleanField(default=False, help_text='Whether admin has indicated the service provider metadata was uploaded.'),
        ),
        migrations.AlterField(
            model_name='historicalenterprisecustomerssoconfiguration',
            name='marked_authorized',
            field=models.BooleanField(default=False, help_text='Whether admin has indicated the service provider metadata was uploaded.'),
        ),
    ]
