# Generated by Django 2.2.17 on 2020-12-15 08:58

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise', '0116_auto_20201208_1759'),
    ]

    operations = [
        migrations.AlterField(
            model_name='enterprisecustomeridentityprovider',
            name='enterprise_customer',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='enterprise_customer_identity_providers', to='enterprise.EnterpriseCustomer'),
        ),
    ]
