# Generated by Django 3.2.20 on 2023-09-28 15:30

from django.db import migrations
import fernet_fields.fields


class Migration(migrations.Migration):

    dependencies = [
        ('moodle', '0027_alter_historicalmoodleenterprisecustomerconfiguration_options'),
    ]

    operations = [
        migrations.AddField(
            model_name='historicalmoodleenterprisecustomerconfiguration',
            name='decrypted_password',
            field=fernet_fields.fields.EncryptedCharField(blank=True, help_text="The encrypted API user's password used to obtain new tokens. It will be encrypted when stored in the database.", max_length=255, null=True, verbose_name='Encrypted Webservice Password'),
        ),
        migrations.AddField(
            model_name='historicalmoodleenterprisecustomerconfiguration',
            name='decrypted_token',
            field=fernet_fields.fields.EncryptedCharField(blank=True, help_text="The encrypted API user's token used to obtain new tokens. It will be encrypted when stored in the database.", max_length=255, null=True, verbose_name='Encrypted Webservice Token'),
        ),
        migrations.AddField(
            model_name='historicalmoodleenterprisecustomerconfiguration',
            name='decrypted_username',
            field=fernet_fields.fields.EncryptedCharField(blank=True, help_text="The encrypted API user's username used to obtain new tokens. It will be encrypted when stored in the database.", max_length=255, null=True, verbose_name='Encrypted Webservice Username'),
        ),
        migrations.AddField(
            model_name='moodleenterprisecustomerconfiguration',
            name='decrypted_password',
            field=fernet_fields.fields.EncryptedCharField(blank=True, help_text="The encrypted API user's password used to obtain new tokens. It will be encrypted when stored in the database.", max_length=255, null=True, verbose_name='Encrypted Webservice Password'),
        ),
        migrations.AddField(
            model_name='moodleenterprisecustomerconfiguration',
            name='decrypted_token',
            field=fernet_fields.fields.EncryptedCharField(blank=True, help_text="The encrypted API user's token used to obtain new tokens. It will be encrypted when stored in the database.", max_length=255, null=True, verbose_name='Encrypted Webservice Token'),
        ),
        migrations.AddField(
            model_name='moodleenterprisecustomerconfiguration',
            name='decrypted_username',
            field=fernet_fields.fields.EncryptedCharField(blank=True, help_text="The encrypted API user's username used to obtain new tokens. It will be encrypted when stored in the database.", max_length=255, null=True, verbose_name='Encrypted Webservice Username'),
        ),
    ]
