# Generated by Django 3.2.12 on 2022-03-10 18:59

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('degreed2', '0007_auto_20220302_2231'),
    ]

    operations = [
        migrations.AddField(
            model_name='degreed2learnerdatatransmissionaudit',
            name='course_completed',
            field=models.BooleanField(default=True, help_text="The learner's course completion status transmitted to Degreed2."),
        ),
    ]
